//******************************************************************************
//******************************************************************************
//*   Copyright (C) 2013 K. Vasudevan. Indian Institute of Technology Kanpur.
//*   All Rights Reserved.
//******************************************************************************
// This code computes the CRB for a given SNR.
// Code tested on SCILAB ver 5.3.3.
//******************************************************************************
clear;
timer();
//******************************************************************************
PRE_AMB_SIZE=512;                                     // Length of the preamble.
OFDM_SIZE=4096;           // Length of OFDM symbol, excluding the cyclic prefix.
CHAN_MEM=9;                                         // Length of channel memory.
CHAN_LEN=CHAN_MEM+1;                                       // Length of channel.
FOFF_EST_RX_SIZE=PRE_AMB_SIZE-CHAN_LEN+1;     // Size of rx. vec. for freq. est.
FADE_VAR=0.5;                               // Variance per dimension of fading.
Pav=2.0;                                // Avg. power of the QPSK constellation.
//******************************************************************************
// SNR parameters.
//******************************************************************************
SNR_DB=12.0;                                                       // SNR in dB.
FADE_MEAN=0.0;                                                // Mean of fading.
FADE_VAR=0.5;                               // Variance per dimension of fading.
FADE_STD=sqrt(FADE_VAR);                            // Std. deviation of fading.
NOISE_MEAN=0.0;                                                 // Mean of AWGN.
NOISE_VAR=CHAN_LEN/(OFDM_SIZE*10^(0.1*SNR_DB));    // Var per dimension of AWGN.
NOISE_STD=sqrt(NOISE_VAR);                           // Std. deviation of noise.
//******************************************************************************
A=[1:FOFF_EST_RX_SIZE];
Re_Fade=grand(A,'nor',FADE_MEAN,FADE_STD);
Im_Fade=grand(A,'nor',FADE_MEAN,FADE_STD);
Fade=Re_Fade+%i*Im_Fade;
B=(A.^2).*abs(Fade).^2;
sum_rand=sum(B);
sum_int=sum(A.^2);
P_Error=100.0*(sum_rand-sum_int)/sum_int;
//******************************************************************************
RMS_Err_FOFF_CRB=2.0*CHAN_LEN*sum_int/(OFDM_SIZE*NOISE_VAR);
RMS_Err_FOFF_CRB=1.0/sqrt(RMS_Err_FOFF_CRB);
timer
//******************************************************************************
