//******************************************************************************
//*   Copyright (C) 2011 K. Vasudevan. Indian Institute of Technology Kanpur.
//*   All Rights Reserved.
//******************************************************************************
// MAP (BCJR) algorithm for decoder 1.
// Code tested on SCILAB ver 5.3.2.
//******************************************************************************
function[Prob]= Dec1_Process(Prob,My_Alpha,My_Beta)
//******************************************************************************
// Initialize alpha and beta.
//******************************************************************************
// my_alpha=zeros(NUM_STATES,OFDM_SIZE);
// my_beta=zeros(NUM_STATES,OFDM_SIZE+1);
 My_Alpha(:,1)=1.0;
 My_Beta(:,OFDM_SIZE+1)=1.0;
//******************************************************************************
// Compute alpha and beta for decoder 1.
//******************************************************************************
 for time=1:OFDM_SIZE-1
//******************************************************************************
// Alpha beta recursion.
//******************************************************************************
//  for state=1:NUM_STATES
//   t_alpha=0.0;
//   t_beta=0.0;
//   for ip=1:NUM_INPUTS
//    prv_st=P_State(state,ip);
//    nxt_st=N_State(state,ip);
//    ipa=P_Ip(state,ip);                          // Input in the form of 1, 2.
//    opa=Op(prv_st,ipa);                    // Encoded bit in the form of 0, 1.
//    opb=Op(state,ip);                      // Encoded bit in the form of 0, 1.
//    inxa=2*(ipa-1)+opa+1;                                    // Range: 1 to 4.
//    inxb=2*(ip-1)+opb+1;                                     // Range: 1 to 4.
//    ga=My_Gamma1(inxa,time);
//    gb=My_Gamma1(inxb,OFDM_SIZE+1-time);
//    proba=Prob(ipa,time);
//    probb=Prob(ip,OFDM_SIZE+1-time);
//    prv_alpha=my_alpha(prv_st,time);
//    nxt_beta=my_beta(nxt_st,OFDM_SIZE+2-time);
//    t_alpha=t_alpha+ga*proba*prv_alpha;
//    t_beta=t_beta+gb*probb*nxt_beta;
//   end
//   my_alpha(state,time+1)=t_alpha;
//   my_beta(state,OFDM_SIZE+1-time)=t_beta;
//  end
//******************************************************************************
// Faster alpha beta recursion.
//******************************************************************************
  My_Alpha(:,time+1)=My_Alpha(P_State(:,1),time).* ...
  My_Gamma1(Ga_Inx(:,1),time).* ...
  Prob(P_Ip(:,1),time)+ ...
  My_Alpha(P_State(:,2),time).* ...
  My_Gamma1(Ga_Inx(:,2),time).* ...
  Prob(P_Ip(:,2),time);
  My_Beta(:,OFDM_SIZE+1-time)=My_Beta(N_State(:,1),OFDM_SIZE+2-time).* ...
  My_Gamma1(Gb_Inx(:,1),OFDM_SIZE+1-time)* ...
  Prob(1,OFDM_SIZE+1-time)+ ...
  My_Beta(N_State(:,2),OFDM_SIZE+2-time).* ...
  My_Gamma1(Gb_Inx(:,2),OFDM_SIZE+1-time)* ...
  Prob(2,OFDM_SIZE+1-time);
  [My_Alpha,My_Beta]=Normalize_Alpha_Beta(My_Alpha,My_Beta,time);
 end
//******************************************************************************
// Compute the a posteriori probabilities (APPs).
//******************************************************************************
// Slow version.
//******************************************************************************
// for time=1:OFDM_SIZE
//  app(1)=0.0;
//  app(2)=0.0;
//  for state=1:NUM_STATES
//   for ip=1:NUM_INPUTS
//    nxt_state=N_State(state,ip);
//    op=Op(state,ip);
//    inx=2*(ip-1)+op+1;
//    ge=My_Gamma1(inx,time);
//    prv_alpha=my_alpha(state,time);
//    nxt_beta=my_beta(nxt_state,time+1);
//    app(ip)=app(ip)+ge*prv_alpha*nxt_beta;
//   end
//  end
//  my_sum=0.0;
//  for ip=1:NUM_INPUTS
//   my_sum=my_sum+app(ip);
//  end
//  for ip=1:NUM_INPUTS
//   Prob(ip,time)=app(ip)/my_sum;
//  end
// end
//******************************************************************************
// Faster version to compute APPs.
//******************************************************************************
 Prob(1,:)=sum(My_Alpha(:,:).*My_Beta(N_State(:,1),2:OFDM_SIZE+1).* ...
 My_Gamma1(Gb_Inx(:,1),:),'r');
 Prob(2,:)=sum(My_Alpha(:,:).*My_Beta(N_State(:,2),2:OFDM_SIZE+1).* ...
 My_Gamma1(Gb_Inx(:,2),:),'r');
 tmp_sum=sum(Prob,'r');
 Prob(1,:)=Prob(1,:)./tmp_sum;
 Prob(2,:)=Prob(2,:)./tmp_sum;
endfunction
