//******************************************************************************
//*   Copyright (C) 2011 K. Vasudevan. Indian Institute of Technology Kanpur.
//*   All Rights Reserved.
//******************************************************************************
// MAP (BCJR) algorithm for decoder 2.
// Code tested on SCILAB ver 5.3.2.
//******************************************************************************
function[Prob]= Dec2_Process_End(Prob)
//******************************************************************************
// Initialize alpha and beta.
//******************************************************************************
 my_alpha=zeros(NUM_STATES,OFDM_SIZE);
 my_beta=zeros(NUM_STATES,OFDM_SIZE+1);
 my_alpha(:,1)=1.0;
 my_beta(:,OFDM_SIZE+1)=1.0;
//******************************************************************************
// Compute alpha and beta for decoder 2.
//******************************************************************************
// for time=1:OFDM_SIZE-1
//  iv_tima=Div_Map(time);
//  iv_timb=Div_Map(OFDM_SIZE+1-time);
//******************************************************************************
// Alpha beta recursion.
//******************************************************************************
//  for state=1:NUM_STATES
//   t_alpha=0.0;
//   t_beta=0.0;
//   for ip=1:NUM_INPUTS
//    prv_st=P_State(state,ip);
//    nxt_st=N_State(state,ip);
//    ipa=P_Ip(state,ip);                          // Input in the form of 1, 2.
//    opa=Op(prv_st,ipa);                    // Encoded bit in the form of 0, 1.
//    opb=Op(state,ip);                      // Encoded bit in the form of 0, 1.
//    inxa=2*(ipa-1)+opa+1;                                    // Range: 1 to 4.
//    inxb=2*(ip-1)+opb+1;                                     // Range: 1 to 4.
//    ga=My_Gamma2(inxa,time);
//    gb=My_Gamma2(inxb,OFDM_SIZE+1-time);
//    proba=Prob(ipa,iv_tima);
//    probb=Prob(ip,iv_timb);
//    prv_alpha=my_alpha(prv_st,time);
//    nxt_beta=my_beta(nxt_st,OFDM_SIZE+2-time);
//    t_alpha=t_alpha+ga*proba*prv_alpha;
//    t_beta=t_beta+gb*probb*nxt_beta;
//   end
//   my_alpha(state,time+1)=t_alpha;
//   my_beta(state,OFDM_SIZE+1-time)=t_beta;
//  end
// end
//******************************************************************************
 My_Initial_Val=ones(2*NUM_STATES,1);
 My_Initial_Time=1;
 time=2:OFDM_SIZE;
 Final_AB_Val=ode('discrete',My_Initial_Val,1,time,Get_AB_Dec2);
 my_alpha(:,time)=Final_AB_Val(1:NUM_STATES,:);
 my_beta(:,OFDM_SIZE:-1:2)=Final_AB_Val(NUM_STATES+1:2*NUM_STATES,:);
//******************************************************************************
// Compute the a posteriori probabilities (APPs).
//******************************************************************************
// Slow version.
//******************************************************************************
// for time=1:OFDM_SIZE
//  app(1)=0.0;
//  app(2)=0.0;
//  iv_tim=Div_Map(time);
//  for state=1:NUM_STATES
//   for ip=1:NUM_INPUTS
//    nxt_state=N_State(state,ip);
//    op=Op(state,ip);
//    inx=2*(ip-1)+op+1;
//    ge=My_Gamma2(inx,time);
//    prv_alpha=my_alpha(state,time);
//    nxt_beta=my_beta(nxt_state,time+1);
//    if iter == NUM_ITER
//     my_prob=Prob(ip,iv_tim);
//    else
//     my_prob=1.0;
//    end
//    app(ip)=app(ip)+ge*prv_alpha*nxt_beta*my_prob;
//   end
//  end
//  my_sum=0.0;
//  for ip=1:NUM_INPUTS
//   my_sum=my_sum+app(ip);
//  end
//  for ip=1:NUM_INPUTS
//   Prob(ip,iv_tim)=app(ip)/my_sum;
//  end
// end
//******************************************************************************
// Faster version to compute APPs.
//******************************************************************************
 Prob(1,Div_Map(:))=sum(my_alpha(:,:).* ...
 my_beta(N_State(:,1),2:OFDM_SIZE+1).* ...
 My_Gamma2(Gb_Inx(:,1),:).* ...
 repmat(Prob(1,Div_Map(:)),NUM_STATES,1),'r');
 Prob(2,Div_Map(:))=sum(my_alpha(:,:).* ...
 my_beta(N_State(:,2),2:OFDM_SIZE+1).* ...
 My_Gamma2(Gb_Inx(:,2),:).* ...
 repmat(Prob(2,Div_Map(:)),NUM_STATES,1),'r');
 tmp_sum=sum(Prob,'r');
 Prob(1,:)=Prob(1,:)./tmp_sum;
 Prob(2,:)=Prob(2,:)./tmp_sum;
endfunction
